import 'dart:math';

import 'package:flutter/material.dart';

import 'src/media_query_observer.dart';

class NestedMediaQuery extends StatelessWidget {
  final Widget child;
  final Widget Function(BuildContext, Size) builder;
  const NestedMediaQuery({
    Key key,
    this.child,
    this.builder,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return _NestedBuilder(
      builder: (context, size) {
        if (builder != null) return builder(context, size);
        if (child != null) return child;
        return Container();
      },
    );
  }
}

class _NestedBuilder extends StatelessWidget {
  final Widget Function(BuildContext, Size) builder;
  const _NestedBuilder({
    Key key,
    @required this.builder,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(
      builder: (context, dimens) {
        final _current = MediaQuery.of(context, nullOk: true);
        final _size = Size(
          min(dimens.maxWidth, _current?.size?.width ?? double.infinity),
          min(dimens.maxHeight, _current?.size?.height ?? double.infinity),
        );
        return MediaQueryObserver(
          data: _current.copyWith(size: _size),
          child: builder(context, _size),
        );
      },
    );
  }
}
